/************************************************************************

(C) Copyright 2004 - Analog Devices, Inc.  All rights reserved.

File Name:		Init_code.c

Author:			Bob Nakib

Date Modified:	08/11/04

Software:       VisualDSP++3.5

Hardware:		ADSP-BF533/532/531

Special Connections:  None

Purpose:		Inflates a compressed user-application image, that has
				been programmed into Flash memory, and runs the inflated
				program from SDRAM.
				
*************************************************************************/

#include "InitHead.h"

#define INIT_ADDR 0x2000001C
#define DXECOUNT_ADDR 0x20000014

/* ----------------------------------------------
The user must set the UNCOMPR_ADDR field to the 
intended destination address for the uncompress() 
function, i.e. the beginning address of the 
section that was 'reserved' or kept empty in 
the SDRAM memory of the user-application.
----------------------------------------------- */

int UNCOMPR_ADDR = 0x4000;

int UNCOMPR_LEN;

void main()
{
	unsigned char *params_addr, *image_addr;
	int dxecount, source_len;
	unsigned long dest_len;
	
	union u			// Can only read from FLASH using 8-bit accesses
	{
		int i;
		char s[4];
	} sample;
	
	*pEBIU_AMGCTL |= 0xE; // Enable all banks
		
	sample.s[0] = *((char *)(DXECOUNT_ADDR));
	sample.s[1] = *((char *)(DXECOUNT_ADDR + 2));	
	sample.s[2] = *((char *)(DXECOUNT_ADDR + 4));
	sample.s[3] = *((char *)(DXECOUNT_ADDR + 6));
	
	dxecount = 2*(sample.i);
	
	params_addr = (unsigned char *)(dxecount + INIT_ADDR);
	
	sample.s[0] = *((char *)(params_addr));
	sample.s[1] = *((char *)(params_addr + 2));	
	sample.s[2] = *((char *)(params_addr + 4));
	sample.s[3] = *((char *)(params_addr + 6));
	
	UNCOMPR_LEN = sample.i;
	dest_len = (unsigned long)UNCOMPR_LEN;
	
	sample.s[0] = *((char *)(params_addr + 8));
	sample.s[1] = *((char *)(params_addr + 10));	
	sample.s[2] = *((char *)(params_addr + 12));
	sample.s[3] = *((char *)(params_addr + 14));
	
	source_len = sample.i;	
	
	image_addr = params_addr + 16;

	SetupSDRAM();

    uncompress((unsigned char *)UNCOMPR_ADDR, &dest_len, image_addr, (unsigned long)source_len);
    
    RestoreSP();
}

