/* Copyright (C) 2000-2004 Analog Devices Inc., All Rights Reserved.
** This contains Analog Devices Background IP and Development IP as
** defined in the ADI/Intel Collaboration Agreement.
*/
// basic startup code which
// - turns the cycle counter on
// - loads up FP & SP (both supervisor and user)
// - initialises the device drivers (FIOCRT)
// - calls monstartup to set up the profiling routines (PROFCRT)
// - calls the C++ startup (CPLUSCRT)
// - initialises argc/argv (FIOCRT/normal)
// - calls _main
// - calls _exit (which calls monexit to dump accumulated prof data (PROFCRT))
// - defines dummy IO routines (!FIOCRT)

#if defined(__ADSPLPBLACKFIN__)
#include <Def_LPBlackfin.h>
#else
#include <DefBlackfin.h>
#endif
#include "cplb.h"
#define IVBh (EVT0 >> 16)
#define IVBl (EVT0 & 0xFFFF)
#define UNASSIGNED_VAL 0x8181
#define UNASSIGNED_FILL 0
// just IVG15
#define INTERRUPT_BITS 0x400
#define SYSCFG_VALUE 0x30
#if defined(__AD6532__) || defined(__ADSPBF535__) || defined(__ADSPBF561__)
#define SET_CLOCK_SPEED 0
#else
#define SET_CLOCK_SPEED 1
#endif

#if SET_CLOCK_SPEED == 1
#include <sys/pll.h>
#define SET_CLK_MSEL 0x16
#define SET_CLK_DF 0
#define SET_CLK_LOCK_COUNT 0x300
#define SET_CLK_CSEL 0
#define SET_CLK_SSEL 5

.global stackp;
/*
** CLKIN == 27MHz on the EZ-Kits.
** D==0 means CLKIN is passed to PLL without dividing.
** MSEL==0x16 means VCO==27*0x16 == 594MHz
** CSEL==0 means CCLK==VCO == 594MHz
** SSEL==5 means SCLK==VCO/5 == 118MHz
*/

#endif

	.section program;
	.align 2;
start:
	[--SP] = ASTAT;			//Save Regs onto stack
	[--SP] = RETS;
	[--SP] = (r7:0);
	[--SP] = (p5:0);
	[--SP] = I0;
	[--SP] = I1;
	[--SP] = I2;
	[--SP] = I3;
	[--SP] = B0;
	[--SP] = B1;
	[--SP] = B2;
	[--SP] = B3;
	[--SP] = M0;
	[--SP] = M1;
	[--SP] = M2;
	[--SP] = M3;
	[--SP] = L0;
	[--SP] = L1;
	[--SP] = L2;
	[--SP] = L3;
	
	P0.H = stackp;
	P0.L = stackp;
	[P0] = SP;
	
#if !defined(__ADSPBF535__) || !defined(__AD6532__)
	// Avoid BF532 Anomaly #42
	R1 = SYSCFG_VALUE;
	SYSCFG = R1;
#endif
	// Zap loop counters to zero, to make sure that
	// hw loops are disabled - it could be really baffling
	// if the counters and bottom regs are set, and we happen
	// to run into them.
	R7 = 0;
	LC0 = R7;
	LC1 = R7;

	// Clear the DAG Length regs too, so that it's safe to
	// use I-regs without them wrapping around.
	L0 = R7;
	L1 = R7;
	L2 = R7;
	L3 = R7;

	// Initialise the Event Vector table.
	P0.H = IVBh;
	P0.L = IVBl;

	// Install __unknown_exception_occurred in EVT so that 
	// there is defined behaviour.
	P0 += 2*4;		// Skip Emulation and Reset
	P1 = 13;
	R1.L = __unknown_exception_occurred;
	R1.H = __unknown_exception_occurred;
	LSETUP (.ivt,.ivt) LC0 = P1;
.ivt:	[P0++] = R1;
	// Set IVG15's handler to be the start of the mode-change
	// code. Then, before we return from the Reset back to user
	// mode, we'll raise IVG15. This will mean we stay in supervisor
	// mode, and continue from the mode-change point., but at a
	// much lower priority.
	P1.H = supervisor_mode;
	P1.L = supervisor_mode;
	[P0] = P1;
	P2.H = ___cplb_ctrl;
	P2.L = ___cplb_ctrl;
	R0 = CPLB_ENABLE_ANY_CPLBS;
	R6 = [P2];
	R0 = R0 & R6;
	CC = R0;
	IF !CC JUMP no_cplb_hdr;
	P1.H = _cplb_hdr;
	P1.L = _cplb_hdr;
	[P0-48] = P1;	// write exception handler
no_cplb_hdr:
	// Initialise the stack.
	// Note: this points just past the end of the section.
	// First write should be with [--SP].
//	SP.L=ldf_stack_end;
//	SP.H=ldf_stack_end;
	usp = sp;

	// We're still in supervisor mode at the moment, so the FP
	// needs to point to the supervisor stack.
	FP = SP;

	// And make space for incoming "parameters" for functions
	// we call from here:
	SP += -12;

	R0 = INTERRUPT_BITS;
	R0 <<= 5;	// Bits 0-4 not settable.
	CALL.X __install_default_handlers;
	R4 = R0;		// Save modified list

	R0 = SYSCFG;		// Enable the Cycle counter
	BITSET(R0,1);
	SYSCFG = R0;

#if !defined(__ADSPBF535__) && !defined(__AD6532__)
	// Set the port preferences of DAG0 and DAG1 to be
	// different; this gives better performance when
	// performing daul-dag operations on SDRAM.
	P0.L = DMEM_CONTROL & 0xFFFF;
	P0.H = DMEM_CONTROL >> 16;
	R0 = [P0];
	BITSET(R0, 12);
	BITCLR(R0, 13);
	[P0] = R0;
	CSYNC;
#endif

#if defined(__ADSPLPBLACKFIN__)
#if SET_CLOCK_SPEED == 1

   // Check if this feature is enabled, i.e. ___clk_ctrl is defined to non-zero
   P0.L = ___clk_ctrl;
   P0.H = ___clk_ctrl;
   R0 = MAX_IN_STARTUP;
   R1 = [P0];
   R0 = R0 - R1;
   CC = R0;
   IF CC JUMP clock_is_set;

	// Investigate whether we are a suitable revision
	// for boosting the system clocks.
	// speed.
	P0.H = 0xFFE0;
	P0.L = 0x5000;
	R0 = [P0];
	R0 <<= 30;
	R0 >>= 30;
	CC = R0 < 2;
	IF CC JUMP clock_is_set;

	// Set the internal Voltage-Controlled Oscillator (VCO)
	R0 = SET_CLK_MSEL (Z);
	R1 = SET_CLK_DF (Z);
	R2 = SET_CLK_LOCK_COUNT (Z);
	CALL.X _pll_set_system_vco;

	// Set the Core and System clocks
	R0 = SET_CLK_CSEL (Z);
	R1 = SET_CLK_SSEL (Z);
	CALL.X _pll_set_system_clocks;

clock_is_set:
#endif
#endif /* ADSPLPBLACKFIN */

	// Reinitialise data areas in RAM from ROM, if MemInit's
	// been used.
	CALL.X _mi_initialize;

#ifdef FIOCRT
	// initialise the devices known about for stdio.
	CALL.X _init_devtab;
#endif /* FIOCRT */

	// initialise the CPLBs if they're needed. couldn't do
	// this before we set up the stacks.
	R0 = CPLB_ENABLE_ANY_CPLBS;
	R0 = R0 & R6;
	CC = R0;
	IF !CC JUMP no_cplb_init;
	R0 = R6;
	CALL.X _cplb_init;
no_cplb_init:

	//  Enable interrupts
	STI R4;		// Using the mask from default handlers
	RAISE 15;

	// Move the processor into user mode.
	P0.L=usermode;
	P0.H=usermode;
	RETI=P0;
	rti;		// should take us to following instruction in user mode
usermode:
	// We don't want to stay in user mode,
	// because of the bloat required to service requests to supervisor
	// resources. So we've arranged for IVG15 to be raised earlier,
	// and it should take effect here...
	JUMP usermode;

	// ...and bring us to here.
supervisor_mode:
	[--SP] = RETI;	// re-enables the interrupt system


	R0.L = UNASSIGNED_VAL;
	R0.H = UNASSIGNED_VAL;
#if UNASSIGNED_FILL
	R2=R0;
	R3=R0;
	R4=R0;
	R5=R0;
	R6=R0;
	R7=R0;
	P0=R0;
	P1=R0;
	P2=R0;
	P3=R0;
	P4=R0;
	P5=R0;
#endif
	// Push a RETS and Old FP onto the stack, for sanity.
	[--SP]=R0;
	[--SP]=R0;
	// Make sure the FP is sensible.
	FP = SP;

	// And leave space for incoming "parameters"
	SP += -12;

#ifdef PROFCRT
        CALL.X monstartup; // initialise profiling routines
#endif  /* PROFCRT */
#ifdef PROFGUIDE
        CALL.X ___start_prof; // initialise profile guided routine
#endif  /* PROFGUIDE */
#ifdef CPLUSCRT
	R0=0;		// load up R0 and R1 and call _main()
	R1=R0;
	CALL.X ___ctorloop;
#endif  /* CPLUSCRT */
#ifdef FIOCRT
	// FILE IO provides access to real command-line arguments.
	CALL.X __getargv;
	r1.l=__Argv; 
	r1.h=__Argv;
#else
	// Default to having no arguments and a null list.
	R0=0;
	R1.L=argv;
	R1.H=argv;
#endif /* FIOCRT */

	// At long last, call the application program.
	CALL.X _main;

	CALL.X _exit;	// passing in main's return value

.start.end:		// To keep the linker happy.


	.global start;
	.type start,STT_FUNC;
	.global .start.end;
	.type .start.end,STT_FUNC;
	.extern _main;
	.type _main,STT_FUNC;
	.extern _exit;
	.type _exit,STT_FUNC;
#ifdef FIOCRT
	.extern __getargv;
	.type __getargv,STT_FUNC;
	.extern _init_devtab;
	.type _init_devtab,STT_FUNC;
	.extern __Argv;
#else
	// If File IO support isn't provided, then
	// we provide dummy versions of the device-handling
	// functions, so that the exception handlers don't rely
	// on the file IO library
	.align 2;
_dev_open:
_dev_close:
_dev_write:
_dev_read:
_dev_seek:
_dev_dup:
	R0 = -1;
	RTS;
._dev_open.end:
._dev_close.end:
._dev_write.end:
._dev_read.end:
._dev_seek.end:
._dev_dup.end:

	.global _dev_open;
	.type _dev_open,STT_FUNC;
	.global _dev_close;
	.type _dev_close,STT_FUNC;
	.global _dev_write;
	.type _dev_write,STT_FUNC;
	.global _dev_read;
	.type _dev_read,STT_FUNC;
	.global _dev_seek;
	.type _dev_seek,STT_FUNC;
	.global _dev_dup;
	.type _dev_dup,STT_FUNC;
#endif /* FIOCRT */


	.section data1;
.byte4 	stackp;
#ifndef FIOCRT
	// With no FILE IO support, we cannot fetch arguments
	// from the command line, so we provide a null list.
	.align 4;
argv:
	.byte4=0;				// argv[0]==0
#endif /* !FIOCRT */
	.align 4;

	.align 4;
nullp:
	.byte4=0;				// no environment pointers
	.align 4;
__Environ:
	.byte4=nullp;
	.global __Environ;
	.type __Environ,STT_OBJECT;

.extern ldf_stack_end;
.extern _mi_initialize;
.type _mi_initialize, STT_FUNC;

.extern _cplb_hdr;
.extern _cplb_init;
.extern ___cplb_ctrl;
.extern __install_default_handlers;
.extern __unknown_exception_occurred;
#if SET_CLOCK_SPEED == 1
.extern ___clk_ctrl;
.extern _pll_set_system_vco;
.extern _pll_set_system_clocks;
#endif

#ifdef PROFCRT
.extern monstartup;
#endif
#ifdef PROFGUIDE
.extern ___start_prof;
#endif

#ifdef CPLUSCRT
.section ctor;
	.align 4;
___ctor_table:
	.byte4=0;				
.global ___ctor_table;
.type ___ctor_table,STT_OBJECT;
.extern ___ctorloop;
.section .gdt;
        .align 4;
___eh_gdt:
.global ___eh_gdt;
        .byte4=0;
.type ___eh_gdt,STT_OBJECT;
.section .frt;
        .align 4;
___eh_frt:
.global ___eh_frt;
        .byte4=0;
.type ___eh_frt,STT_OBJECT;
#endif  /* CPLUSCRT */
